#!perl

use Test::More tests => 1;
use Config qw(%Config);

BEGIN {
    use_ok( 'Sys::GetRandom' );
}

diag( "Testing Sys::GetRandom $Sys::GetRandom::VERSION, Perl $], $^X" );
diag("\$^O: $^O; archname: $Config{archname}; SYS_getrandom: " . Sys::GetRandom::SYS_getrandom . "; GRND_RANDOM: " . Sys::GetRandom::GRND_RANDOM . "; GRND_NONBLOCK: " . Sys::GetRandom::GRND_NONBLOCK);
